package com.cooleshow.teacher.ui.homepage;

import android.view.View;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.LinearLayoutManager;

import com.alibaba.android.arouter.launcher.ARouter;
import com.chad.library.adapter.base.BaseQuickAdapter;
import com.chad.library.adapter.base.listener.OnItemClickListener;
import com.chad.library.adapter.base.listener.OnLoadMoreListener;
import com.cooleshow.base.common.WebConstants;
import com.cooleshow.base.constanst.Constants;
import com.cooleshow.base.router.RouterPath;
import com.cooleshow.base.widgets.EmptyViewLayout;
import com.cooleshow.teacher.R;
import com.cooleshow.teacher.adapter.SparringCourseListAdapter;
import com.cooleshow.teacher.bean.SparringCourseListBean;
import com.cooleshow.teacher.contract.SparringCoursePageContract;
import com.cooleshow.teacher.databinding.FragmentSparringCoursePageLayoutBinding;
import com.cooleshow.teacher.presenter.homePage.SparringCoursePagePresenter;
import com.cooleshow.teacher.ui.course.SparringCourseDetailActivity;
import com.cooleshow.base.ui.fragment.BaseMVPFragment;
import com.scwang.smart.refresh.layout.api.RefreshLayout;
import com.scwang.smart.refresh.layout.listener.OnRefreshListener;

/**
 * Author by pq, Date on 2022/5/9.
 */
public class SparringCoursePageFragment extends BaseMVPFragment<FragmentSparringCoursePageLayoutBinding, SparringCoursePagePresenter> implements SparringCoursePageContract.SparringCoursePageView, View.OnClickListener {
    private int currentPage;
    private boolean hasNext = true;
    private SparringCourseListAdapter mAdapter;
    private EmptyViewLayout mEmptyView;
    @Override
    protected void initView(View rootView) {
        mViewBinding.llPracticeSet.setOnClickListener(this);
    }

    @Override
    protected void initData() {
        mViewBinding.refreshLayout.setOnRefreshListener(new OnRefreshListener() {
            @Override
            public void onRefresh(@NonNull RefreshLayout refreshLayout) {
                currentPage = 1;
                queryCourse();
            }
        });

        mAdapter = new SparringCourseListAdapter(R.layout.item_sparring_course_list_layout);
        mAdapter.getLoadMoreModule().setOnLoadMoreListener(new OnLoadMoreListener() {
            @Override
            public void onLoadMore() {
                //上拉加载
                if (hasNext) {
                    currentPage++;
                    queryCourse();
                } else {
                    mAdapter.getLoadMoreModule().loadMoreEnd();
                }
            }
        });
        mAdapter.setOnItemClickListener(new OnItemClickListener() {
            @Override
            public void onItemClick(@NonNull BaseQuickAdapter<?, ?> adapter, @NonNull View view, int position) {
                if (position < mAdapter.getData().size()) {
                    SparringCourseListBean.RowsBean rowsBean = mAdapter.getData().get(position);
                    if (rowsBean != null) {
                        ARouter.getInstance()
                                .build(RouterPath.CourseCenter.SPARRING_COURSE_DETAIL)
                                .withString(SparringCourseDetailActivity.COURSE_ID, rowsBean.courseId)
                                .withString(SparringCourseDetailActivity.COURSE_GROUP_ID, rowsBean.courseGoupId)
                                .withString(SparringCourseDetailActivity.STUDENT_ID, rowsBean.userId)
                                .navigation();
                    }
                }
            }
        });
        mViewBinding.recyclerView.setLayoutManager(new LinearLayoutManager(requireContext()));
        mViewBinding.recyclerView.setAdapter(mAdapter);
        currentPage = 1;
        queryCourse();
    }

    @Override
    protected FragmentSparringCoursePageLayoutBinding getLayoutView() {
        return FragmentSparringCoursePageLayoutBinding.inflate(getLayoutInflater());
    }

    @Override
    protected SparringCoursePagePresenter createPresenter() {
        return new SparringCoursePagePresenter();
    }

    @Override
    public void onClick(View view) {
        switch (view.getId()){
            case R.id.ll_practice_set:
                ARouter.getInstance()
                        .build(RouterPath.WebCenter.ACTIVITY_HTML)
                        .withString(WebConstants.WEB_URL, WebConstants.TEACHER_PRACTICE_SETTING)
                        .navigation();
                break;
        }
    }
    private void queryCourse() {
        //根据默认筛选条件查询
        presenter.querySparringCourse( currentPage);
    }

    @Override
    public void onGetSparringCourseSuccess(int page, SparringCourseListBean sparringCourseBean) {
        if (isDetached()) {
            return;
        }
        if (sparringCourseBean != null) {
            if (page == 1) {
                //第一页
                mViewBinding.refreshLayout.finishRefresh();
                if (mAdapter != null) {
                    mAdapter.getData().clear();
                    mAdapter.notifyDataSetChanged();
                    if (sparringCourseBean.rows != null && sparringCourseBean.rows.size() > 0) {
                        checkHasNext(sparringCourseBean.rows.size());
                        mAdapter.setNewInstance(sparringCourseBean.rows);
                    } else {
                        showEmptyView();
                    }
                }
            } else {
                //加载更多
                if (mAdapter != null) {
                    if (sparringCourseBean.rows != null && sparringCourseBean.rows.size() > 0) {
                        mAdapter.getLoadMoreModule().loadMoreComplete();
                        checkHasNext(sparringCourseBean.rows.size());
                        mAdapter.addData(sparringCourseBean.rows);
                    } else {
                        mAdapter.getLoadMoreModule().loadMoreEnd();
                    }
                }
            }
        }
    }
    private void showEmptyView() {
        if (mEmptyView == null) {
            mEmptyView = new EmptyViewLayout(getContext());
        }
        mEmptyView.setContent(com.cooleshow.base.R.drawable.icon_empty_course, "暂无课程~");
        mAdapter.setEmptyView(mEmptyView);
    }
    private void checkHasNext(int dataSize) {
        hasNext = dataSize >= Constants.DEFAULT_DATA_SIZE;
    }

    @Override
    public void onGetCourseError(int page) {
        if (isDetached()) {
            return;
        }
        if (page == 1) {
            mViewBinding.refreshLayout.finishRefresh();
        } else {
            if (mAdapter != null) {
                currentPage--;
                mAdapter.getLoadMoreModule().loadMoreFail();
            }
        }
    }
}
